package services

import (
	"eta/eta_api/models"
	"eta/eta_api/utils"
	"fmt"
	"time"
)

// MoveEnPermission 移动品种
func MoveEnPermission(req models.EnPermissionMoveReq) (err error, errMsg string) {
	ob := new(models.EnPermission)
	permissionId := req.PermissionId
	prevPermissionId := req.PrevPermissionId
	nextPermissionId := req.NextPermissionId

	//如果有传入 上一个兄弟节点分类id
	var (
		permissionInfo *models.EnPermission
		prevPermission *models.EnPermission
		nextPermission *models.EnPermission

		prevSort int
		nextSort int
	)

	// 移动对象为分类， 判断权限
	permissionInfo, err = ob.GetItemById(permissionId)
	if err != nil {
		if err.Error() == utils.ErrNoRow() {
			errMsg = "品种不存在, 请刷新页面"
			err = fmt.Errorf("获取品种信息失败,Err:" + err.Error())
			return
		}
		errMsg = "移动失败"
		err = fmt.Errorf("获取分类信息失败,Err:" + err.Error())
		return
	} else if permissionInfo.EnPermissionId == 0 {
		errMsg = "品种不存在, 请刷新页面"
		err = fmt.Errorf("获取品种信息失败,Err:" + err.Error())
		return
	}

	parentPermissionId := permissionInfo.ParentId
	if prevPermissionId > 0 {
		prevPermission, err = ob.GetItemById(prevPermissionId)
		if err != nil {
			if err.Error() == utils.ErrNoRow() {
				errMsg = "上一个品种不存在, 请刷新页面"
				err = fmt.Errorf("获取品种信息失败,Err:" + err.Error())
				return
			}
			errMsg = "移动失败"
			err = fmt.Errorf("获取上一个兄弟节点分类信息失败,Err:" + err.Error())
			return
		}
		prevSort = prevPermission.Sort
	}

	if nextPermissionId > 0 {
		//下一个兄弟节点
		nextPermission, err = ob.GetItemById(nextPermissionId)
		if err != nil {
			if err.Error() == utils.ErrNoRow() {
				errMsg = "下一个品种不存在, 请刷新页面"
				err = fmt.Errorf("获取品种信息失败,Err:" + err.Error())
				return
			}
			errMsg = "移动失败"
			err = fmt.Errorf("获取下一个兄弟节点分类信息失败,Err:" + err.Error())
			return
		}
		nextSort = nextPermission.Sort
	}

	err, errMsg = moveEnPermission(permissionInfo, prevPermission, nextPermission, parentPermissionId, prevSort, nextSort)
	return
}

// moveEnPermission 移动品种
func moveEnPermission(permissionInfo, prevPermission, nextPermission *models.EnPermission, parentId, prevSort, nextSort int) (err error, errMsg string) {
	ob := new(models.EnPermission)
	updateCol := make([]string, 0)

	//判断上级id是否一致，如果不一致的话，那么需要移动该分类层级
	if permissionInfo.ParentId != parentId {
		errMsg = "移动失败"
		err = fmt.Errorf("不支持目录层级变更")
		return
	}

	if prevSort > 0 {
		//如果是移动在两个兄弟节点之间
		if nextSort > 0 {
			//下一个兄弟节点
			//如果上一个兄弟与下一个兄弟的排序权重是一致的，那么需要将下一个兄弟（以及下个兄弟的同样排序权重）的排序权重+2，自己变成上一个兄弟的排序权重+1
			if prevSort == nextSort || prevSort == permissionInfo.Sort {
				//变更兄弟节点的排序
				updateSortStr := `sort + 2`

				//变更分类
				if prevPermission != nil {
					_ = models.UpdateEnPermissionSortByParentId(parentId, prevPermission.EnPermissionId, prevPermission.Sort, updateSortStr)
				} else {
					_ = models.UpdateEnPermissionSortByParentId(parentId, 0, prevSort, updateSortStr)
				}

			} else {
				//如果下一个兄弟的排序权重正好是上个兄弟节点的下一层，那么需要再加一层了
				if nextSort-prevSort == 1 {
					//变更兄弟节点的排序
					updateSortStr := `sort + 1`

					//变更分类
					if prevPermission != nil {
						_ = models.UpdateEnPermissionSortByParentId(parentId, prevPermission.EnPermissionId, prevSort, updateSortStr)
					} else {
						_ = models.UpdateEnPermissionSortByParentId(parentId, 0, prevSort, updateSortStr)
					}

				}
			}
		}

		permissionInfo.Sort = prevSort + 1
		permissionInfo.ModifyTime = time.Now()
		updateCol = append(updateCol, "Sort", "ModifyTime")
	} else if prevPermission == nil && nextPermission == nil && parentId > 0 {
		//处理只拖动到目录里，默认放到目录底部的情况
		var maxSort int
		maxSort, err = ob.GetMaxSortByParentId(parentId)
		if err != nil {
			errMsg = "移动失败"
			err = fmt.Errorf("查询组内排序信息失败,Err:" + err.Error())
			return
		}
		permissionInfo.Sort = maxSort + 1 //那就是排在组内最后一位
		permissionInfo.ModifyTime = time.Now()
		updateCol = append(updateCol, "Sort", "ModifyTime")
	} else {
		// 拖动到父级分类的第一位
		firstPermission, tmpErr := ob.GetFirstEnPermissionByParentId(parentId)
		if tmpErr != nil && tmpErr.Error() != utils.ErrNoRow() {
			errMsg = "移动失败"
			err = fmt.Errorf("获取获取当前父级分类下的排序第一条的分类信息失败,Err:" + tmpErr.Error())
			return
		}

		//如果该分类下存在其他分类，且第一个其他分类的排序等于0，那么需要调整排序
		if firstPermission != nil && firstPermission.EnPermissionId != 0 && firstPermission.Sort == 0 {
			updateSortStr := ` sort + 1 `
			_ = models.UpdateEnPermissionSortByParentId(parentId, firstPermission.EnPermissionId-1, 0, updateSortStr)
		}

		permissionInfo.Sort = 0 //那就是排在第一位
		permissionInfo.ModifyTime = time.Now()
		updateCol = append(updateCol, "Sort", "ModifyTime")
	}

	//更新
	if len(updateCol) > 0 {
		err = permissionInfo.Update(updateCol)
		if err != nil {
			errMsg = "移动失败"
			err = fmt.Errorf("修改失败,Err:" + err.Error())
			return
		}
	}
	return
}
